/*
 * $Id: Bulletsmorph.java,v 1.1.1.1 2002/07/07 10:27:57 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.bulletsmorph;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

/**
 * Bulletsmorph bootstrap.
 *
 * @version $Revision: 1.1.1.1 $
 */
public class Bulletsmorph extends Applet {

  private float rank;
  private String dtdUrl;

  private GPEngine engine, initialEngine;

  private BulletmlPlayer player;
  private BulletsCanvas bulletsCanvas;
  private Screen screen;

  private String baseFileName;
  private String[] fileName;

  private boolean standAlone = false;

  private void usage() {
    System.err.println(
     "Usage: bulletmorph [-rank n] [-dtdurl URL]");
  }

  private void parseArgs(String[] args) throws IOException {
    for ( int i=0 ; i<args.length ; i++ ) {
      if ( args[i].equals("-rank") ) {
        rank = (float)Double.parseDouble(args[i+1]); i++;
      } else if ( args[i].equals("-dtdurl") ) {
        dtdUrl = args[i+1]; i++;
      } else {
        throw new IOException("Invalid args.");
      }
    }
  }

  private void initEngines() {
    try {
      GPEngine.init(dtdUrl, rank);
      player = new BulletmlPlayer();
      engine = new GPEngine();
      screen = new Screen();
      bulletsCanvas = new BulletsCanvas(
       this, 0, engine.getBulletmlManager(), player, engine, screen, rank);
      bulletsCanvas.setSize(Screen.WIDTH, Screen.HEIGHT);
      engine.loadCatalogBulletml(fileName);
      engine.loadBaseBulletml(baseFileName);

      initialEngine = new GPEngine();
      initialEngine.copyEngine(engine);

      engine.initGeneration();
      engine.gotoNextGeneration();
    } catch ( Exception e ) {
      e.printStackTrace();
    }
  }

  private void resetEngines() {
    engine.copyEngine(initialEngine);
    engine.initGeneration();
    engine.gotoNextGeneration();
  }

  private void this_mousePressed(MouseEvent e) {
    if ( e.getY() > 328 ) {
      if ( e.getX() < 292 ) {
        generationFinished();
      } else {
        resetAll();
      }
    }
  }

  public void start() {
    if ( !standAlone ) {
      appletInit();
    }
    initEngines();
    setLayout(new FlowLayout());
    add(bulletsCanvas);
    this.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        this_mousePressed(e);
      }
    });
    repaint();
  }

  public void stop() {
    if ( player != null ) player.terminate();
  }

  public void destroy() {
    if ( player != null ) player.terminate();
  }

  public void paint(Graphics g) {
    Dimension dim = getSize();
    int w = dim.width, h = dim.height;
    g.setColor(new Color(255, 255, 255));
    g.fillRect(0, 0, w, h);
    g.setColor(new Color(160, 160, 160));
    for ( int x = 0 ; x<w ; x += 36 ) {
      g.drawLine(x, 0, x, h);
    }
    for ( int y = 0 ; y<h ; y += 36 ) {
      g.drawLine(0, y, w, y);
    }
    Color lc = new Color(0, 0, 0);
    LetterRender.drawString(g, "generation " + engine.getGeneration(), 8, 332, 9, lc);
    LetterRender.drawString(g, "reset", 292, 340, 6, lc);

    bulletsCanvas.repaint();
  }

  public void update(Graphics g) {
    paint(g);
  }

  public void generationFinished() {
    bulletsCanvas.reset();
    engine.gotoNextGeneration();
    repaint();
    bulletsCanvas.start();
  }

  public void resetAll() {
    bulletsCanvas.reset();
    resetEngines();
    repaint();
    bulletsCanvas.start();
  }

  private void appletInit() {
    dtdUrl = "http://www.asahi-net.or.jp/~cs8k-cyu/bulletml/bulletml.dtd";
    rank = 0.5f;

    baseFileName = BulletmlData.BASE_FILE_NAME;
    fileName = BulletmlData.getFileNames();
    GPEngine.readFromFile = false;
 }

  public void standAloneInit(String[] args) throws IOException {
    standAlone = true;

    dtdUrl = "file:///c:./bulletml.dtd";
    rank = 0.5f;
    try {
      parseArgs(args);
    } catch ( Exception e ) {
      usage();
      return;
    }
    baseFileName = BulletmlData.BASE_FILE_NAME;
    File dir = new File("bulletmls/");
    fileName = dir.list();
    for ( int i=0 ; i<fileName.length ; i++ ) {
      fileName[i] = "bulletmls/" + fileName[i];
    }
    GPEngine.readFromFile = true;

    start();
    Frame frame = new Frame("Bulletmorph");
    frame.setSize(360, 360+24);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(1);
      }
    });
    frame.setLayout(new BorderLayout());
    frame.add(this, BorderLayout.CENTER);
    frame.setVisible(true);
  }

  public static void main(String[] args) {
    try {
      new Bulletsmorph().standAloneInit(args);
    } catch ( IOException e ) {
      e.printStackTrace();
    }
  }
}
