/*
 * $Id: GPEngine.java,v 1.1.1.1 2002/07/07 10:27:57 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.bulletsmorph;

import jp.gr.java_conf.abagames.bulletml.*;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import java.io.*;
import java.util.*;

/**
 * Bullets genetic programing engine.
 *
 * @version $Revision: 1.1.1.1 $
 */
public class GPEngine {
  private static String dtdUrl;
  private static float rank;

  public static void init(String du, float rk) {
    dtdUrl = du;
    rank = rk;
  }


  private BulletmlManager manager = new BulletmlManager();

  private int insertActionsNum, generation;

  public GPEngine() {
    manager.initBulletmlUtil();
    manager.setGPEngine(this);
  }

  public void initGeneration() {
    insertActionsNum = 1;
    generation = 0;
  }

  public int getGeneration() {
    return generation;
  }

  public BulletmlManager getBulletmlManager() {
    return manager;
  }

  public Vector actionsCatalog = new Vector();

  private IActionElmChoice[] loadBulletML(String document, String fileName) {
    try {
      ErrorHandler errorHandler = new ErrorHandler() {
        public void error(SAXParseException e) {
          e.printStackTrace();
        }
        public void fatalError(SAXParseException e) {
          e.printStackTrace();
        }
        public void warning(SAXParseException e) {
          e.printStackTrace();
        }
      };
      Document doc = UJAXP.getValidDocument(
       new StringReader(document), errorHandler);
      Bulletml bulletML = new Bulletml(doc);
      IBulletmlChoice[] bmc = bulletML.getContent();
      Vector aecVct = new Vector();
      for ( int i=0 ; i<bmc.length ; i++ ) {
        IBulletmlChoice be = bmc[i];
        if ( be instanceof Action ) {
          Action act = (Action)be;
          if ( act.getLabel().startsWith("top") ) {
            aecVct.addElement(act);
          }
          manager.bulletmlUtil.addAction(act, fileName);
        } else if ( be instanceof Bullet ) {
          manager.bulletmlUtil.addBullet((Bullet)be, fileName);
        } else if ( be instanceof Fire ) {
          manager.bulletmlUtil.addFire((Fire)be, fileName);
        }
      }
      IActionElmChoice[] topAction = new IActionElmChoice[aecVct.size()];
      aecVct.copyInto(topAction);
      return topAction;
    } catch ( Exception e ) {
      e.printStackTrace();
      return null;
    }
  }

  public static boolean readFromFile;

  private String getDTDReplacedStr(String fn)
   throws FileNotFoundException, IOException {
    // Replace DTD URL.
    String doc;
    if ( readFromFile ) {
      StringBuffer buf = new StringBuffer();
      File file = new File(fn);
      BufferedReader reader = new BufferedReader(new FileReader(file));
      for ( ; ; ) {
        String ln = reader.readLine();
        if ( ln == null ) break;
        buf.append(ln);
      }
      doc = buf.toString();
    } else {
      doc = BulletmlData.getContext(fn);
    }
    if ( dtdUrl != null ) {
      int dtSt = doc.indexOf("<!DOCTYPE");
      int dtEd = doc.indexOf(">", dtSt+1) + 1;
      doc = doc.substring(0, dtSt) +
       "<!DOCTYPE bulletml SYSTEM \"" + dtdUrl + "\">" +
       doc.substring(dtEd);
    }
    return doc;
  }

  private IActionElmChoice[] processAllActions(String fileName) throws IOException {
    IActionElmChoice[] actions =
     loadBulletML(getDTDReplacedStr(fileName), fileName);

    manager.init();
    manager.setFileName(fileName);
    manager.addTopBullet(actions, Screen.WIDTH<<7, Screen.HEIGHT<<6, 180, rank);
    while ( !manager.isFinished() ) {
      manager.moveBullets();
    }
    return actions;
  }

  public void loadCatalogBulletml(String[] fileName) throws IOException {
    actionsCatalog = new Vector();
    manager.setActionsCatalog(actionsCatalog);
    manager.bulletmlUtil.setActionImplsVector(actionsCatalog);
    manager.bulletmlUtil.setTopBulletEnable(false);
    manager.bulletmlUtil.addActionToNormBullet = false;
    for ( int i=0 ; i<fileName.length ; i++ ) {
      processAllActions(fileName[i]);
    }
  }


  private IActionElmChoice[] baseActions;
  private Vector baseActionImpls;

  public void loadBaseBulletml(String baseFileName) throws IOException {
    baseActionImpls = new Vector();
    manager.bulletmlUtil.setActionImplsVector(baseActionImpls);
    manager.bulletmlUtil.setTopBulletEnable(true);
    manager.bulletmlUtil.addActionToNormBullet = true;
    baseActions = processAllActions(baseFileName);
  }

  public IActionElmChoice[] getBaseActions() {
    return baseActions;
  }

  public Vector getBaseActionImpls() {
    return baseActionImpls;
  }

  private Vector copyActionImplVector(Vector v2) {
    Vector v1 = new Vector();
    Enumeration elms = v2.elements();
    while ( elms.hasMoreElements() ) {
      v1.addElement(new ActionImpl((ActionImpl)elms.nextElement()));
    }
    return v1;
  }

  public void copyEngine(GPEngine engine) {
    IActionElmChoice[] aec = engine.getBaseActions();
    baseActions = new IActionElmChoice[aec.length];
    try {
      for ( int i=0 ; i<aec.length ; i++ ) {
        if ( aec[i] instanceof Action ) {
          baseActions[i] = new Action(((Action)aec[i]).makeDocument());
        } else if ( aec[i] instanceof ActionRef ) {
          baseActions[i] = new ActionRef(((ActionRef)aec[i]).makeDocument());
        }
      }
    } catch ( Exception e ) {
      e.printStackTrace();
      return;
    }

    actionsCatalog = copyActionImplVector(engine.actionsCatalog);
    manager.setActionsCatalog(actionsCatalog);

    baseActionImpls = new Vector();
    manager.bulletmlUtil.setActionImplsVector(baseActionImpls);
    manager.bulletmlUtil.setTopBulletEnable(true);
    manager.bulletmlUtil.addActionToNormBullet = true;
    manager.init();
    manager.setFileName(null);
    manager.addTopBullet(baseActions, Screen.WIDTH<<7, Screen.HEIGHT<<6, 180, rank);
    while ( !manager.isFinished() ) {
      manager.moveBullets();
    }
    manager.bulletmlUtil = new BulletmlUtil(engine.manager.bulletmlUtil);
  }

  private Random rnd = new Random();

  public static final String IMPL_LABEL = "impl:";

  public void insertCatalogAction() {
    Vector bac = baseActionImpls;
    int bacn = Math.abs(rnd.nextInt())%(bac.size());
    ActionImpl act = (ActionImpl)bac.elementAt(bacn);
    IActionChoice[] aec = act.action;
    ActionRef ar = new ActionRef();

    Vector cac = actionsCatalog;
    int cacn = Math.abs(rnd.nextInt())%cac.size();

    ar.setLabel(IMPL_LABEL + cacn);

    IActionChoice[] newAec = new IActionChoice[aec.length+1];
    //int naecIdx = rnd.nextInt()%(aec.length+1);
    int naecIdx;
    if ( aec.length <= 0 ) {
      naecIdx = 0;
    } else {
      if ( aec[aec.length-1] instanceof Vanish ) {
        naecIdx = rnd.nextInt()%(aec.length);
      } else {
        naecIdx = rnd.nextInt()%(aec.length+1);
      }
    }
    int i;
    for ( i=0 ; i<naecIdx; i++ ) {
      newAec[i] = aec[i];
    }
    newAec[i] = ar; i++;
    for ( ; i<newAec.length ; i++ ) {
      newAec[i] = aec[i-1];
    }
    act.actionElm.setContent(newAec);
  }

  public void insertCatalogActionFinished() {
    baseActionImpls = new Vector();
    manager.bulletmlUtil.setActionImplsVector(baseActionImpls);
    manager.bulletmlUtil.setTopBulletEnable(false);
    manager.bulletmlUtil.addActionToNormBullet = true;

    manager.setActionsCatalog(actionsCatalog);

    manager.init();
    manager.setFileName(null);
    manager.addTopBullet(
     getBaseActions(), Screen.WIDTH<<7, Screen.HEIGHT<<6, 180, rank);
    while ( !manager.isFinished() ) {
      manager.moveBullets();
    }
    if ( baseActionImpls.size() <= 0 ) {
      baseActionImpls = new Vector();
      manager.bulletmlUtil.setActionImplsVector(baseActionImpls);
      manager.bulletmlUtil.setTopBulletEnable(true);
      manager.bulletmlUtil.addActionToNormBullet = true;
      manager.init();
      manager.setFileName(null);
      manager.addTopBullet(
       getBaseActions(), Screen.WIDTH<<7, Screen.HEIGHT<<6, 180, rank);
      while ( !manager.isFinished() ) {
        manager.moveBullets();
      }
    }
  }

  public void gotoNextGeneration() {
    manager.init();
    for ( int i=0 ; i<insertActionsNum ; i++ ) {
      insertCatalogAction();
    }
    insertCatalogActionFinished();
    insertActionsNum++;
    generation++;
    manager.init();
    manager.rewind();
  }
}
